# -*- coding: utf-8 -*-

import pandas as pd
import requests
import json
from cachetools import cached

class Client(object):
    """
    Buffett Code API Client

    Attributes:
        apikey (str): Apikey for BuffettCode API
    """

    ENDPOINT = "https://api.buffett-code.com"

    def _get(self, path, params={}):
        headers = {"x-api-key": self.apikey}
        return requests.get(
            f"{Client.ENDPOINT}/api/v2{path}", params=params, headers=headers
        )

    def __init__(self, apikey):
        self.apikey = apikey

    @cached(cache={})
    def quarter(self, ticker, from_q, to_q):
        """Quarter endpoint

        See: http://docs.buffett-code.com/#/default/get_api_v2_quarter

        Parameters:
          ticker (str): Ticker symbol
          from_q (str): Beginning quarter of the target range
          to_q (str): End quarter of the target range

        """
        res = self._get("/quarter", {"tickers": ticker, "from": from_q, "to": to_q})
        j = res.json()
        return j

    @cached(cache={})
    def companies(self):
        """Company endpoint

        See: http://docs.buffett-code.com/#/default/get_api_v2_company
        """
        res = self._get("/company")
        j = res.json()
        return j

    @cached(cache={})
    def search(self, keywords):
        """Search endpoint

        See: http://docs.buffett-code.com/#/default/get_api_v2_search
        """
        res = self._get("/search", {"keywords": keywords})
        j = res.json()
        return j



# -*- coding: utf-8 -*-

import pandas as pd
from cachetools import cached

from .client import Client


class Indicator(object):
    """
    Financial indicators for the given company
    """

    def __init__(self, apikey):
        self.client = Client(apikey=apikey)

    @cached(cache={})
    def get(self, ticker, from_q, to_q):
        q = self.client.quarter(ticker, from_q, to_q)
        # c = self.client.company(ticker)
        return pd.DataFrame(q[ticker])



# -*- coding: utf-8 -*-

from .client import Client
from .company import Company
from .company import Stockprice
import quandl


class Omaha(object):
    """Omaha provides a unified view of the financial information of the public companies.

    Attributes:
        bc_apikey (str): Apikey of BuffettCode API
        quandl_apikey (str): Apikey of Quandl
    """

    def __init__(self, bc_apikey, quandl_apikey):
        self.client = Client(bc_apikey)
        quandl.ApiConfig.api_key = quandl_apikey

    def company(self, ticker, from_q, to_q):
        """Basic financial indicators of the company

        Args:
            ticker (str): Ticker symbol
            from_q (str): Beginning of the target quarter
            to_q (str): End of the target quarter

        Returns:
            Company: Target company

        Example:
            >>> factory = Omaha(bc_apikey='xxxx'), quandl_apikey='yyyy')
            >>> factory.company('1111', '2019Q1', '2019Q4')
        """
        return Company(ticker, from_q, to_q, self.client)

    def stockprice(self, ticker, start_date, end_date):
        """Stockprice of the given company

        Args:
            ticker (str): Ticker symbol
            start_date (str): Start date of the target range
            end_date (str): End date of the target range

        Returns:
            Stockprice: Object containing stock prices

        Example:
            >>> factory = Omaha(bc_apikey='xxxx'), quandl_apikey='yyyy')
            >>> factory.stockprice('1111', '2019-01-01', '2019-12-31')
        """
        return Stockprice(ticker, start_date, end_date)

    def search(self, keywords, from_q, to_q):
        """List of matched companies with given search keyword

        Args:
            keywords (str): Keyword used for searching companies
            from_q (str): Beginning quarter of the target range
            to_q (str): End quarter of the target range

        Returns:
          list: List of companies matching with the given keyword.

        Example:
            >>> factory = Omaha(bc_apikey='xxxx'), quandl_apikey='yyyy')
            >>> factory.search('不動産', '2019-01-01', '2019-12-31')
        """
        tickers = self.client.search(keywords)
        return [self.company(ticker, from_q, to_q) for ticker in tickers]

    def category(self, cat, from_q, to_q):
        """List of companies filtering by the given TSE 33 sectors.

        See: https://www.jpx.co.jp/english/markets/indices/line-up/files/e_fac_13_sector.pdf

        Args:
            keywords (str): Keyword used for searching companies
            from_q (str): Beginning quarter of the target range
            to_q (str): End quarter of the target range

        Returns:
          list: List of companies matching with the given TSE 33 sectors.

        Example:
            >>> factory = Omaha(bc_apikey='xxxx'), quandl_apikey='yyyy')
            >>> factory.category('サービス業', '2019-01-01', '2019-12-31')
        """
        companies = self.client.companies()
        tickers = []
        for ticker, company in companies.items():
            if ticker == 'column_description':
                continue
            if company[0]['tosyo_33category'] == cat:
                tickers.append(ticker)

        return [self.company(ticker, from_q, to_q) for ticker in tickers]





# -*- coding: utf-8 -*-

import pandas as pd


class Joinable(object):
    """Joinable views

    The dataframe of the class can be joined with the end date of the financial report.
    """

    def __init__(self, joinables):
        self.joinables = joinables

    def join(self, other):
        """Joined with other financial data container.
        """
        return Joinable(self.joinables + other.joinables)

    def raw_df(self):
        pass

    def df(self):
        """DataFrame representing the underlying financial data.

        Returns:
            DataFrame: DataFrame representing the underlying financial data
        """
        dfs = [j.raw_df() for j in self.joinables]
        return pd.concat(dfs, axis=1, join="inner")



# -*- coding: utf-8 -*-

import unittest
from unittest.mock import MagicMock
import requests_mock

from omaha.company import Company
from omaha.client import Client


class CompanyTest(unittest.TestCase):
    def setUp(self):
        mock_client = Client(apikey="xxx")
        mock_client.quarter = MagicMock(
            return_value={
                "column_description": {
                    "company_name": {"name_jp": "会社名", "unit": "なし"}
                },
                "1111": [
                    {
                        "company_name": "海山商事",
                        "fiscal_year": "2019",
                        "fiscal_quarter": "4",
                    },
                    {
                        "company_name": "海山商事",
                        "fiscal_year": "2019",
                        "fiscal_quarter": "3",
                    },
                ],
            }
        )
        self.company = Company("1111", "2019Q1", "2019Q4", mock_client)

    def test_dict_pairs(self):
        res = Company.dict_pairs({"a": 1, "b": 2, "c": 3}, ["a", "b"])
        self.assertEqual({"a": 1, "b": 2}, res)

    def test_company_name(self):
        expected = [
            {"company_name": "海山商事", "fiscal_quarter": "4", "fiscal_year": "2019"},
            {"company_name": "海山商事", "fiscal_quarter": "3", "fiscal_year": "2019"},
        ]
        self.assertEqual(expected, self.company.get("company_name"))



# -*- coding: utf-8 -*-

from cachetools import cached
from dateutil import parser
import pandas as pd
import quandl

from omaha.joinable import Joinable


class Company(Joinable):
    """Container for the financial indicators of the public company

    Attributes:
        ticker (str): Ticker symbol
        from_q (str): Beginning quarter of the target range
        to_q (str): End quarter of the target range
        client (Client): BuffettCode API Client
    """
    def __init__(self, ticker, from_q, to_q, client):
        self.ticker = ticker
        self.client = client
        self.from_q = from_q
        self.to_q = to_q
        super().__init__([self])

    @classmethod
    def dict_pairs(cls, d, keys):
        return {k: v for k, v in d.items() if k in keys}

    @cached(cache={})
    def __get(self, from_q, to_q):
        return self.client.quarter(self.ticker, from_q, to_q)

    def __str__(self):
        return f"Company({self.ticker}, {self.from_q}, {self.to_q})"

    def __repr__(self):
        return self.__str__()

    def get(self, item):
        res = self.__get(self.from_q, self.to_q)
        keys = [item, "fiscal_year", "fiscal_quarter"]
        return [Company.dict_pairs(d, keys) for d in res[self.ticker]]

    def all(self):
        res = self.__get(self.from_q, self.to_q)
        return res[self.ticker]

    def raw_df(self):
        res = self.__get(self.from_q, self.to_q)
        df = pd.DataFrame(res[self.ticker])
        index = [pd.Timestamp(s, tz="UTC") for s in df["end_date"]]
        df.index = index
        return df


class Stockprice(Joinable):
    """Container for the daily stockprice of the public company.
    """
    def __init__(self, ticker, start_date, end_date):
        self.ticker = ticker
        self.start_date = start_date
        self.end_date = end_date
        super().__init__([self])

    def raw_df(self):
        df = quandl.get(
            f"XJPX/{self.ticker}0", start_date=self.start_date, end_date=self.end_date
        )
        df.index = [pd.Timestamp(s, tz="UTC") for s in df.index]
        return df



# -*- coding: utf-8 -*-

import unittest
import requests_mock

from omaha.indicator import Indicator


class IndicatorTest(unittest.TestCase):
    def setUp(self):
        self.indicator = Indicator(apikey="apikey")

    def test_quarter(self):
        with requests_mock.Mocker() as m:
            response = {
                "1111": [{"company_name": "海山商事"}],
                "column_description": {
                    "company_name": {"name_jp": "会社名", "unit": "なし"}
                },
            }
            m.get(
                "https://api.buffett-code.com/api/v2/quarter?tickers=1111&from=2019Q1&to=2019Q4",
                json=response,
            )
            df = self.indicator.get("1111", "2019Q1", "2019Q4")
            self.assertEqual(df.columns, ["company_name"])



# -*- coding: utf-8 -*-

import unittest
from unittest.mock import MagicMock
import requests_mock

from omaha.company import Company
from omaha.client import Client


class JoinableTest(unittest.TestCase):
    def setUp(self):
        mock_client1 = Client(apikey="xxx")
        mock_client1.quarter = MagicMock(
            return_value={
                "column_description": {
                    "company_name": {"name_jp": "会社名", "unit": "なし"}
                },
                "1111": [
                    {
                        "end_date": "2019-01-01",
                        "company_name": "海山商事",
                        "fiscal_year": "2019",
                        "fiscal_quarter": "4",
                    },
                    {
                        "end_date": "2019-01-02",
                        "company_name": "海山商事",
                        "fiscal_year": "2019",
                        "fiscal_quarter": "3",
                    },
                ],
            }
        )
        self.company1 = Company("1111", "2019Q1", "2019Q4", mock_client1)
        mock_client2 = Client(apikey="xxx")
        mock_client2.quarter = MagicMock(
            return_value={
                "column_description": {
                    "company_name": {"name_jp": "会社名", "unit": "なし"}
                },
                "2222": [
                    {
                        "end_date": "2019-01-01",
                        "company_name": "海山商事",
                        "fiscal_year": "2019",
                        "fiscal_quarter": "4",
                    },
                    {
                        "end_date": "2019-01-03",
                        "company_name": "海山商事",
                        "fiscal_year": "2019",
                        "fiscal_quarter": "3",
                    },
                ],
            }
        )
        self.company2 = Company("2222", "2019Q1", "2019Q4", mock_client2)

    def test_join(self):
        joined = self.company1.join(self.company2)
        df = joined.df()
        self.assertEqual(8, df.size)
        self.assertEqual(1, df.index.size)
